package Reporting;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(debug_report failed_report ok_report start_report command_report initial_report report_and_die);

use strict;

# uses values set in the configuration section ($log_to_njs, main::CMD_SOCK, $debug)

#
# Logging subs.
#
# Destination of log entries can be either stdout or
# the NJS (plus syslog before NJS connections made)
#
# More verbose messaging if "debug" set
#

# Write a string to the current output with a time stamp (internal)
#
# arg1: string to write

sub report {

    my $time = scalar localtime;
    my $string = shift;

    $string =~ s/\s+$//;

    if($main::log_to_njs) {
        print "TSI_COMMENT\n$time $string\nEND_TSI_COMMENT\n";
    }
    else {
        print "$time $string\n";
    }

}

#
# Write debug information to current output (if debug requested)
#
# arg1: string to write
#
sub debug_report {
        my $string = shift;
        report("     $string");
}

#
# Execution of an NJS request failed, tell NJS with appropriate protocol
# and also write the message to current output
#
# arg1: failure message
#
sub failed_report {

    my $string = shift;

    # NJS expects a single line, remove any embedded new lines
    $string =~ s/\n/:/g;

    report(" ### Command execution FAILED");
    report("     $string");
    report("     Command was: $Reporting::reported_command") if $Reporting::reported_command;
    report(" ### End report command execution FAILED");

    # Tell NJS
    print "TSI_FAILED: $string\n";

}

#
# Execution of an NJS request executed OK, tell NJS with appropriate protocol,
# and also write the message to current output (if debugging)
#
# arg1: debug message 
#
sub ok_report {

    my $string = shift;

    report(" --- Command executed OK");
    debug_report("$string");

    # Tell NJS
    print "TSI_OK\n";

}

#
# Start of execution of a request from NJS, write to current output
#
# arg1: message 
#
sub start_report {
    my $string = shift;
    report(" --- $string");
}

#
# This is the command that was executed, report if debugging
# record for possible printing by a failed command
#
# arg1: the executed command
#
sub command_report {
    $Reporting::reported_command = shift;  # keep this for use by failed_report
    debug_report("Command is: $Reporting::reported_command");
}

#
# Information to log at the start of execution, before
# a connection is made to NJS so to syslog (one day) or stdout.
# (cannot go to NJS because we have not contacted one yet)
#
# arg1: message to write
#
sub initial_report {

    my $string = shift;
    my $time = scalar localtime;

        print "$time     $string\n";
}

#
# Something has happened to make this execution
# die (assumed that NJS has gone so will log to
# syslog). Should not ever be called.
#
# arg1: message to write
#
sub report_and_die {
    my $string = shift;
    initial_report($string);
    exit 1;
}

# $Id: Reporting.pm,v 1.1.1.1 2006/09/08 12:19:37 bschuller Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                       Copyright 1999 to 2001 Fujitsu European Centre for Information Technology Ltd.
#  This is unpublished proprietary source code which is the property of Fujitsu European Centre for Information Technology Ltd.
#
